@@ -10,8 +10,8 @@ Usage:
 from __future__ import absolute_import, division
 # compatibility functions and utilities
 from .utils import _supports_unicode, _environ_cols_wrapper, _range, _unich, \
-    _term_move_up, _unicode, WeakSet, _basestring, _OrderedDict, _text_width, \
-    Comparable, RE_ANSI, _is_ascii, FormatReplace, \
+    _term_move_up, _unicode, WeakSet, _basestring, _OrderedDict, \
+    Comparable, _is_ascii, FormatReplace, disp_len, disp_trim, \
     SimpleTextIOWrapper, CallbackIOWrapper
 from ._monitor import TMonitor
 # native libraries
@@ -479,12 +479,13 @@ class tqdm(Comparable):
             # Formatting progress bar space available for bar's display
             full_bar = Bar(
                 frac,
-                max(1, ncols - _text_width(RE_ANSI.sub('', nobar)))
+                max(1, ncols - disp_len(nobar))
                 if ncols else 10,
                 charset=Bar.ASCII if ascii is True else ascii or Bar.UTF)
             if not _is_ascii(full_bar.charset) and _is_ascii(bar_format):
                 bar_format = _unicode(bar_format)
-            return bar_format.format(bar=full_bar, **format_dict)
+            res = bar_format.format(bar=full_bar, **format_dict)
+            return disp_trim(res, ncols) if ncols else res
 
         elif bar_format:
             # user-specified bar_format but no total
@@ -496,10 +497,11 @@ class tqdm(Comparable):
                 return nobar
             full_bar = Bar(
                 0,
-                max(1, ncols - _text_width(RE_ANSI.sub('', nobar)))
+                max(1, ncols - disp_len(nobar))
                 if ncols else 10,
                 charset=Bar.BLANK)
-            return bar_format.format(bar=full_bar, **format_dict)
+            res = bar_format.format(bar=full_bar, **format_dict)
+            return disp_trim(res, ncols) if ncols else res
         else:
             # no total: no progressbar, ETA, just progress stats
             return ((prefix + ": ") if prefix else '') + \
@@ -656,10 +658,13 @@ class tqdm(Comparable):
             from pandas import Panel
         except ImportError:  # TODO: pandas>0.25.2
             Panel = None
-        try:  # pandas>=0.18.0
-            from pandas.core.window import _Rolling_and_Expanding
-        except ImportError:  # pragma: no cover
-            _Rolling_and_Expanding = None
+        try:  # pandas>=1.0.0
+            from pandas.core.window.rolling import _Rolling_and_Expanding
+        except ImportError:
+            try:  # pandas>=0.18.0
+                from pandas.core.window import _Rolling_and_Expanding
+            except ImportError:  # pragma: no cover
+                _Rolling_and_Expanding = None
         try:  # pandas>=0.25.0
             from pandas.core.groupby.generic import DataFrameGroupBy, \
                 SeriesGroupBy  # , NDFrameGroupBy
@@ -732,7 +737,10 @@ class tqdm(Comparable):
                         " Use keyword arguments instead.",
                         fp_write=getattr(t.fp, 'write', sys.stderr.write))
 
-                func = df._is_builtin_func(func)
+                try:
+                    func = df._is_builtin_func(func)
+                except TypeError:
+                    pass
 
                 # Define bar updating wrapper
                 def wrapper(*args, **kwargs):
